@@ -1,5 +1,5 @@
 # encoding=UTF-8
-# Copyright © 2008, 2009, 2010 Jakub Wilk <ubanus@users.sf.net>
+# Copyright © 2008, 2009, 2010 Jakub Wilk <jwilk@jwilk.net>
 # Copyright © 2009 Mateusz Turcza <mturcza@mimuw.edu.pl>
 #
 # This package is free software; you can redistribute it and/or modify
@@ -61,8 +61,8 @@ WxDjVuMessage, wx.EVT_DJVU_MESSAGE = wx.lib.newevent.NewEvent()
 system_encoding = locale.getpreferredencoding()
 
 if wx.__version__.startswith('2.8.'):
-    # XXX wxWidgets 2.8 seems to crash with wx.Choicebook
     wx.Choicebook = wx.Notebook
+    wx.EVT_CHOICEBOOK_PAGE_CHANGED = wx.EVT_NOTEBOOK_PAGE_CHANGED
 
 class OpenDialog(wx.FileDialog):
 
@@ -70,10 +70,14 @@ class OpenDialog(wx.FileDialog):
         wx.FileDialog.__init__(self, parent, style = wx.OPEN, wildcard=DJVU_WILDCARD, message = _('Open a DjVu document'))
 
 class TextModel(models.text.Text):
+
     def __init__(self, document):
         models.text.Text.__init__(self)
         self._document = document
 
+    def reset_document(self, document):
+        self._document = document
+
     def acquire_data(self, n):
         text = self._document.pages[n].text
         text.wait()
@@ -85,6 +89,9 @@ class OutlineModel(models.outline.Outline):
         self._document = document
         models.outline.Outline.__init__(self)
 
+    def reset_document(self, document):
+        self._document = document
+
     def acquire_data(self):
         outline = self._document.outline
         outline.wait()
@@ -133,6 +140,9 @@ class AnnotationsModel(models.annotations.Annotations):
         models.annotations.Annotations.__init__(self)
         self.__djvused = StreamEditor(document_path)
 
+    def reset_document(self, document):
+        pass # Nothing to do
+
     def acquire_data(self, n):
         djvused = self.__djvused
         if n == models.SHARED_ANNOTATIONS_PAGENO:
@@ -147,10 +157,14 @@ class AnnotationsModel(models.annotations.Annotations):
             raise # FIXME
 
 class MetadataModel(models.metadata.Metadata):
+
     def __init__(self, document):
         models.metadata.Metadata.__init__(self)
         self._document = document
 
+    def reset_document(self, document):
+        self._document = document
+
     def acquire_data(self, n):
         document_annotations = self._document.annotations
         document_annotations.wait()
@@ -331,9 +345,6 @@ class MainWindow(wx.Frame):
                 self.on_display_maparea,
                 self.on_display_text)
         )
-        sidebar_sizer = wx.BoxSizer(wx.VERTICAL)
-        self.sidebar.SetSizer(sidebar_sizer)
-        sidebar_sizer.Add(self.text_browser, 1, wx.ALL | wx.EXPAND)
         self.scrolled_panel = ScrolledPanel(self.splitter)
         self.splitter.SetSashGravity(0.1)
         self.do_show_sidebar()
@@ -455,7 +466,7 @@ class MainWindow(wx.Frame):
             (_('&Previous page') + '\tPgUp',   _('Jump to previous document page'), self.on_previous_page, wx.ART_GO_UP),
             (_('&Next page') + '\tPgDn',       _('Jump to next document page'),     self.on_next_page,     wx.ART_GO_DOWN),
             (_('&Last page') + '\tCtrl-End',   _('Jump to last document page'),     self.on_last_page,     None),
-            (_(u'&Go to page…'),               _(u'Jump to page…'),                 self.on_goto_page,     None)
+            (_(u'&Go to page…') + '\tCtrl-G',  _(u'Jump to page…'),                 self.on_goto_page,     None)
         ]:
             self._menu_item(menu, caption, help, method, icon = icon)
         return menu
@@ -562,6 +573,9 @@ class MainWindow(wx.Frame):
         def job():
             try:
                 sed.commit()
+                self.document = self.context.new_document(djvu.decode.FileURI(self.path))
+                for model in self.models:
+                    model.reset_document(self.document)
             except Exception, exception:
                 pass
             else:
@@ -945,8 +959,8 @@ class MainWindow(wx.Frame):
 
 class Context(djvu.decode.Context):
 
-    def __new__(self, window):
-        return djvu.decode.Context.__new__(self)
+    def __new__(cls, window):
+        return djvu.decode.Context.__new__(cls)
 
     def __init__(self, window):
         djvu.decode.Context.__init__(self)
